package VaxVoIP.VaxUserAgentLib.VaxUserAgentSIP;

import android.content.Context;
import android.util.Log;
import VaxVoIP.VaxUserAgentLib.VaxUserAgentSIP.AudioDevice.AudioDevice;
import VaxVoIP.VaxUserAgentLib.VaxUserAgentSIP.Network.Network;
import VaxVoIP.VaxUserAgentLib.VaxUserAgentSIP.UserAgentSO.VaxUserAgentSO;

public class VaxUserAgentSIP extends VaxUserAgentSO
{
    static
    {
        try
        {
            System.loadLibrary("avcodec");
            System.loadLibrary("avfilter");
            System.loadLibrary("avformat");
            System.loadLibrary("avutil");
            System.loadLibrary("swresample");
            System.loadLibrary("swscale");

            System.loadLibrary("crypto");
            System.loadLibrary("ssl");
        }
        catch (UnsatisfiedLinkError e)
        {
            System.err.println("public native code library failed to load.\n" + e);
            Log.e("Tag",e.getMessage());
        }
    }

    private IVaxUserAgentSIP m_iVaxUserAgentSIP;

    private AudioDevice m_objAudioDevice = null;
    private Network m_objNetwork = null;

    public VaxUserAgentSIP(IVaxUserAgentSIP iVaxUserAgentSIP, Context objContext)
    {
        m_iVaxUserAgentSIP = iVaxUserAgentSIP;

        m_objAudioDevice = new AudioDevice(objContext);
        m_objNetwork = new Network(objContext);
    }

    public boolean DialCall(String sAccountName, int nLineNo, String sCallerName, String sCallerId, String sDialNo, int nInputDeviceId, int nOutputDeviceId)
    {
        if(!super.DialCall(sAccountName, nLineNo, sCallerName, sCallerId, sDialNo, nInputDeviceId, nOutputDeviceId))
            return false;

        m_objAudioDevice.AdjustCommunicationMode();
        return true;
    }

    public boolean AcceptCall(int nLineNo, String sCallId, int nInputDeviceId, int nOutputDeviceId)
    {
        if(!super.AcceptCall(nLineNo, sCallId, nInputDeviceId, nOutputDeviceId))
            return false;

        m_objAudioDevice.AdjustCommunicationMode();
        return true;
    }

    ////////////////////////////////////////////////////////////////////
    /////////////////////////   EVENTS   ///////////////////////////////
    ////////////////////////////////////////////////////////////////////

    @Override
    protected void OnInitialized()
    {
        m_iVaxUserAgentSIP.OnInitialized();
        m_objAudioDevice.OnInitialized();
    }

    @Override
    protected void OnUnInitialized()
    {
        m_objAudioDevice.OnUnInitialized();
        m_iVaxUserAgentSIP.OnUnInitialized();
    }

    @Override
    protected void OnConnectingToRegister(String sAccountName)
    {
        m_iVaxUserAgentSIP.OnConnectingToRegister(sAccountName);
    }

    @Override
    protected void OnTryingToRegister(String sAccountName)
    {
        m_iVaxUserAgentSIP.OnTryingToRegister(sAccountName);
    }

    @Override
    protected void OnFailToRegister(String sAccountName, int nStatusCode, String sReasonPhrase)
    {
        m_iVaxUserAgentSIP.OnFailToRegister(sAccountName, nStatusCode, sReasonPhrase);
    }

    @Override
    protected void OnSuccessToRegister(String sAccountName)
    {
        m_iVaxUserAgentSIP.OnSuccessToRegister(sAccountName);
    }

    @Override
    protected void OnConnectingToReRegister(String sAccountName)
    {
        m_iVaxUserAgentSIP.OnConnectingToReRegister(sAccountName);
    }

    @Override
    protected void OnTryingToReRegister(String sAccountName)
    {
        m_iVaxUserAgentSIP.OnTryingToReRegister(sAccountName);
    }

    @Override
    protected void OnFailToReRegister(String sAccountName, int nStatusCode, String sReasonPhrase)
    {
        m_iVaxUserAgentSIP.OnFailToReRegister(sAccountName, nStatusCode, sReasonPhrase);
    }

    @Override
    protected void OnSuccessToReRegister(String sAccountName)
    {
        m_iVaxUserAgentSIP.OnSuccessToReRegister(sAccountName);
    }

    @Override
    protected void OnTryingToUnRegister(String sAccountName)
    {
        m_iVaxUserAgentSIP.OnTryingToUnRegister(sAccountName);
    }

    @Override
    protected void OnFailToUnRegister(String sAccountName)
    {
        m_iVaxUserAgentSIP.OnFailToUnRegister(sAccountName);
    }

    @Override
    protected void OnSuccessToUnRegister(String sAccountName)
    {
        m_iVaxUserAgentSIP.OnSuccessToUnRegister(sAccountName);
    }

    /////////////////////////////////////////////////////////////////////////////////////////
    /////////////////////////////////////////////////////////////////////////////////////////

    @Override
    protected void OnDialCallStarted(String sAccountName, int nLineNo, String sCallerName, String sCallerId, String sDialNo)
    {
        m_iVaxUserAgentSIP.OnDialCallStarted(sAccountName, nLineNo, sCallerName, sCallerId, sDialNo);
    }

    @Override
    protected void OnDialingCall(String sAccountName, int nLineNo, int nStatusCode, String sReasonPhrase)
    {
        m_iVaxUserAgentSIP.OnDialingCall(sAccountName, nLineNo, nStatusCode, sReasonPhrase);
    }

    @Override
    protected void OnDialCallFailed(String sAccountName, int nLineNo, int nStatusCode, String sReasonPhrase, String sContact)
    {
        m_iVaxUserAgentSIP.OnDialCallFailed(sAccountName, nLineNo, nStatusCode, sReasonPhrase, sContact);
    }

    @Override
    protected void OnConnectedCall(String sAccountName, int nLineNo, String sToAddrRTP, int nToPortRTP)
    {
        m_iVaxUserAgentSIP.OnConnectedCall(sAccountName, nLineNo, sToAddrRTP, nToPortRTP);
    }

    @Override
    protected void OnHungupCall(String sAccountName, int nLineNo)
    {
        m_iVaxUserAgentSIP.OnHungupCall(sAccountName, nLineNo);
    }

    @Override
    protected void OnIncomingCallStarted(String sAccountName, String sCallId, String sCallerName, String sCallerId, String sDialNo, String sFromURI, String sToURI)
    {
        m_iVaxUserAgentSIP.OnIncomingCallStarted(sAccountName, sCallId, sCallerName, sCallerId, sDialNo, sFromURI, sToURI);
    }

    @Override
    protected void OnIncomingCallEnded(String sAccountName, String sCallId, int nReasonCode)
    {
        m_iVaxUserAgentSIP.OnIncomingCallEnded(sAccountName, sCallId, nReasonCode);
    }

    /////////////////////////////////////////////////////////////////////////////////////////
    /////////////////////////////////////////////////////////////////////////////////////////

    @Override
    protected void OnTransferCallFailed(String sAccountName, int nLineNo, int nStatusCode, String sReasonPhrase)
    {
        m_iVaxUserAgentSIP.OnTransferCallFailed(sAccountName, nLineNo, nStatusCode, sReasonPhrase);
    }

    @Override
    protected void OnTransferCallAccepted(String sAccountName, int nLineNo)
    {
        m_iVaxUserAgentSIP.OnTransferCallAccepted(sAccountName, nLineNo);
    }

    @Override
    protected void OnPlayWaveDone(String sAccountName, int nLineNo)
    {
        m_iVaxUserAgentSIP.OnPlayWaveDone(sAccountName, nLineNo);
    }

    @Override
    protected void OnDigitDTMF(String sAccountName, int nLineNo, String sDigit)
    {
        m_iVaxUserAgentSIP.OnDigitDTMF(sAccountName, nLineNo, sDigit);
    }

    @Override
    protected void OnMsgNOTIFY(String sAccountName, String sMsg)
    {
        m_iVaxUserAgentSIP.OnMsgNOTIFY(sAccountName, sMsg);
    }

    @Override
    protected void OnVoiceMailMsg(String sAccountName, boolean bIsMsgWaiting, int nNewMsgCount, int nOldMsgCount, int nNewUrgentMsgCount, int nOldUrgentMsgCount, String sMsgAccount)
    {
        m_iVaxUserAgentSIP.OnVoiceMailMsg(sAccountName, bIsMsgWaiting, nNewMsgCount, nOldMsgCount, nNewUrgentMsgCount, nOldUrgentMsgCount, sMsgAccount);
    }

    @Override
    protected void OnRingToneStarted(String sAccountName, String sCallId)
    {
        m_iVaxUserAgentSIP.OnRingToneStarted(sAccountName, sCallId);
    }

    @Override
    protected void OnRingToneEnded(String sAccountName, String sCallId)
    {
        m_iVaxUserAgentSIP.OnRingToneEnded(sAccountName, sCallId);
    }

    @Override
    protected void OnIncomingDiagnostic(String sAccountName, String sMsgSIP, String sFromIP, int nFromPort)
    {
        m_iVaxUserAgentSIP.OnIncomingDiagnostic(sAccountName, sMsgSIP, sFromIP, nFromPort);
    }

    @Override
    protected void OnOutgoingDiagnostic(String sAccountName, String sMsgSIP, String sToIP, int nToPort)
    {
        m_iVaxUserAgentSIP.OnOutgoingDiagnostic(sAccountName, sMsgSIP, sToIP, nToPort);
    }

    @Override
    protected void OnAudioSessionLost(String sAccountName, int nLineNo)
    {
        m_iVaxUserAgentSIP.OnAudioSessionLost(sAccountName, nLineNo);
    }

    @Override
    protected void OnSuccessToHold(String sAccountName, int nLineNo)
    {
        m_iVaxUserAgentSIP.OnSuccessToHold(sAccountName, nLineNo);
    }

    @Override
    protected void OnTryingToHold(String sAccountName, int nLineNo)
    {
        m_iVaxUserAgentSIP.OnTryingToHold(sAccountName, nLineNo);
    }

    @Override
    protected void OnFailToHold(String sAccountName, int nLineNo)
    {
        m_iVaxUserAgentSIP.OnFailToHold(sAccountName, nLineNo);
    }

    @Override
    protected void OnSuccessToUnHold(String sAccountName, int nLineNo)
    {
        m_iVaxUserAgentSIP.OnSuccessToUnHold(sAccountName, nLineNo);
    }

    @Override
    protected void OnTryingToUnHold(String sAccountName, int nLineNo)
    {
        m_iVaxUserAgentSIP.OnTryingToUnHold(sAccountName, nLineNo);
    }

    @Override
    protected void OnFailToUnHold(String sAccountName, int nLineNo)
    {
        m_iVaxUserAgentSIP.OnFailToUnHold(sAccountName, nLineNo);
    }

    @Override
    protected void OnTryingToRefresh(String sAccountName, int nLineNo)
    {
        m_iVaxUserAgentSIP.OnTryingToRefresh(sAccountName, nLineNo);
    }

    @Override
    protected void OnFailToRefresh(String sAccountName, int nLineNo)
    {
        m_iVaxUserAgentSIP.OnFailToRefresh(sAccountName, nLineNo);
    }

    @Override
    protected void OnSuccessToRefresh(String sAccountName, int nLineNo)
    {
        m_iVaxUserAgentSIP.OnSuccessToRefresh(sAccountName, nLineNo);
    }


    @Override
    protected void OnChatSubscribeSuccess(String sAccountName, String sUserName)
    {
        m_iVaxUserAgentSIP.OnChatSubscribeSuccess(sAccountName, sUserName);
    }

    @Override
    protected void OnChatSubscribeFailed(String sAccountName, String sUserName, int nStatusCode, String sReasonPhrase)
    {
        m_iVaxUserAgentSIP.OnChatSubscribeFailed(sAccountName, sUserName, nStatusCode, sReasonPhrase);
    }

    @Override
    protected void OnChatContactStatus(String sAccountName, String sUserName, int nStatusId)
    {
        m_iVaxUserAgentSIP.OnChatContactStatus(sAccountName, sUserName, nStatusId);
    }

    @Override
    protected void OnChatSendMsgTextSuccess(String sAccountName, String sUserName, String sMsgText, int nUserValue)
    {
        m_iVaxUserAgentSIP.OnChatSendMsgTextSuccess(sAccountName, sUserName, sMsgText, nUserValue);
    }

    @Override
    protected void OnChatSendMsgTextFailed(String sAccountName, String sUserName, int nStatusCode, String sReasonPhrase, String sMsgText, int nUserValue)
    {
        m_iVaxUserAgentSIP.OnChatSendMsgTextFailed(sAccountName, sUserName, nStatusCode, sReasonPhrase, sMsgText, nUserValue);
    }

    @Override
    protected void OnChatSendMsgTypingSuccess(String sAccountName, String sUserName, int nUserValue)
    {
        m_iVaxUserAgentSIP.OnChatSendMsgTypingSuccess(sAccountName, sUserName, nUserValue);
    }

    @Override
    protected void OnChatSendMsgTypingFailed(String sAccountName, String sUserName, int nStatusCode, String sReasonPhrase, int nUserValue)
    {
        m_iVaxUserAgentSIP.OnChatSendMsgTypingFailed(sAccountName, sUserName, nStatusCode, sReasonPhrase, nUserValue);
    }

    @Override
    protected void OnChatRecvMsgText(String sAccountName, String sUserName, String sMsgText, boolean bIsChatContact)
    {
        m_iVaxUserAgentSIP.OnChatRecvMsgText(sAccountName, sUserName, sMsgText, bIsChatContact);
    }

    @Override
    protected void OnChatRecvMsgTypingStart(String sAccountName, String sUserName)
    {
        m_iVaxUserAgentSIP.OnChatRecvMsgTypingStart(sAccountName, sUserName);
    }

    @Override
    protected void OnChatRecvMsgTypingStop(String sAccountName, String sUserName)
    {
        m_iVaxUserAgentSIP.OnChatRecvMsgTypingStop(sAccountName, sUserName);
    }

    @Override
    protected void OnAudioRecordedPCM(String sAccountName, int nLineNo, byte[] aDataPCM, int nSizePCM, int nTypePCM)
    {
        m_iVaxUserAgentSIP.OnAudioRecordedPCM(sAccountName, nLineNo, aDataPCM, nSizePCM, nTypePCM);
    }

    @Override
    protected void OnHoldCall(String sAccountName, int nLineNo)
    {
        m_iVaxUserAgentSIP.OnHoldCall(sAccountName, nLineNo);
    }

    @Override
    protected void OnUnHoldCall(String sAccountName, int nLineNo)
    {
        m_iVaxUserAgentSIP.OnUnHoldCall(sAccountName, nLineNo);
    }

    @Override
    protected void OnVideoRemoteStarted(String sAccountName, int nLineNo)
    {
        m_iVaxUserAgentSIP.OnVideoRemoteStarted(sAccountName, nLineNo);
    }

    @Override
    protected void OnVideoRemoteEnded(String sAccountName, int nLineNo)
    {
        m_iVaxUserAgentSIP.OnVideoRemoteEnded(sAccountName, nLineNo);
    }

    @Override
    protected void OnVideoDeviceFrameRGB(int nDeviceId, byte[] aFrameRGB, int nFrameSize, int nFrameWidth, int nFrameHeight)
    {
        m_iVaxUserAgentSIP.OnVideoDeviceFrameRGB(nDeviceId, aFrameRGB, nFrameSize, nFrameWidth, nFrameHeight);
    }

    @Override
    protected void OnVideoRemoteFrameRGB(String sAccountName, int nLineNo, byte[] aFrameRGB, int nFrameSize, int nFrameWidth, int nFrameHeight)
    {
        m_iVaxUserAgentSIP.OnVideoRemoteFrameRGB(sAccountName, nLineNo, aFrameRGB, nFrameSize, nFrameWidth, nFrameHeight);
    }

    @Override
    protected void OnAddCallHistory(String sAccountName, boolean bOutboundCallType, String sCallerName, String sCallerId, String sDialNo, long nStartTime, long nEndTime, long nDuration, long nDayNo, int nHistoryTypeId)
    {
        m_iVaxUserAgentSIP.OnAddCallHistory(sAccountName, bOutboundCallType, sCallerName, sCallerId, sDialNo, nStartTime, nEndTime, nDuration, nDayNo, nHistoryTypeId);
    }

    @Override
    protected void OnNetworkReachability(boolean bAvailable)
    {
        m_iVaxUserAgentSIP.OnNetworkReachability(bAvailable);
    }

    @Override
    protected void OnAudioDeviceMicVU(int nLevelVU)
    {
        m_iVaxUserAgentSIP.OnAudioDeviceMicVU(nLevelVU);
    }

    @Override
    protected void OnAudioDeviceSpkVU(int nLevelVU)
    {
        m_iVaxUserAgentSIP.OnAudioDeviceSpkVU(nLevelVU);
    }

    @Override
    protected void OnBusyLampSubscribeSuccess(String sAccountName, String sUserName)
    {
        m_iVaxUserAgentSIP.OnBusyLampSubscribeSuccess(sAccountName, sUserName);
    }

    @Override
    protected void OnBusyLampSubscribeFailed(String sAccountName, String sUserName, int nStatusCode, String sReasonPhrase)
    {
        m_iVaxUserAgentSIP.OnBusyLampSubscribeFailed(sAccountName, sUserName, nStatusCode, sReasonPhrase);
    }

    @Override
    protected void OnBusyLampContactStatus(String sAccountName, String sUserName, int nStatusId)
    {
        m_iVaxUserAgentSIP.OnBusyLampContactStatus(sAccountName, sUserName, nStatusId);
    }

    @Override
    protected void OnVaxErrorMsg(String sFuncName, String sErrorMsg, int nErrorCode)
    {
        m_iVaxUserAgentSIP.OnVaxErrorMsg(sFuncName, sErrorMsg, nErrorCode);
    }


}

