package com.vaxsoft.vaxphonesip.VaxPhoneSIP.VaxPhoneAll;

import com.vaxsoft.vaxphonesip.VaxPhoneUI.AccountLogin.AccountLoginActivity;
import com.vaxsoft.vaxphonesip.VaxPhoneUI.MainTab.CallTab.CallTabFragment;
import com.vaxsoft.vaxphonesip.VaxPhoneUI.MainTab.CallTab.DialpadFragment;

import com.vaxsoft.vaxphonesip.VaxUserAgentSIP.IVaxUserAgentSIP;
import com.vaxsoft.vaxphonesip.VaxUserAgentSIP.VaxUserAgentSIP;

public class VaxPhoneStatus implements IVaxUserAgentSIP
{
    private VaxUserAgentSIP mVaxAgentSIP = null;
    boolean m_bVideoCaptureErrorShown = false;

    public VaxPhoneStatus()
    {
        mVaxAgentSIP = new VaxUserAgentSIP(this);
    }

    //////////////////////////////////////////////////////////////////////////////////////////
    //////////////////////////////////////////////////////////////////////////////////////////

    private void PostStatusMsg(String sMsgText)
    {
        AccountLoginActivity.PostStatusText(sMsgText);
        CallTabFragment.PostStatusText(sMsgText);
        DialpadFragment.PostStatusText(sMsgText);
    }

    //////////////////////////////////////////////////////////////////////////////////////////
    //////////////////////////////////////////////////////////////////////////////////////////

    public void OnInitialized()
    {
        PostStatusMsg("Account is online");
    }

    public void OnUnInitialized()
    {
        m_bVideoCaptureErrorShown = false;

        PostStatusMsg("Account is offline");
    }

    //////////////////////////////////////////////////////////////////////////////////////
    //////////////////////////////////////////////////////////////////////////////////////

    public void OnConnectingToRegister(String sAccountName)
    {
        PostStatusMsg("Trying to connect");
    }

    public void OnTryingToRegister(String sAccountName)
    {
        PostStatusMsg("Registering Account");
    }

    public void OnFailToRegister(String sAccountName, int nStatusCode, String sReasonPhrase)
    {
        PostStatusMsg("Register: " + sReasonPhrase);
    }

    public void OnSuccessToRegister(String sAccountName)
    {
        PostStatusMsg("Account Registered");
    }

    //////////////////////////////////////////////////////////////////////////////////////
    /////////////////////////////////////////////////////////////////////////////////////

    public void OnConnectingToReRegister(String sAccountName)
    {

    }

    public void OnTryingToReRegister(String sAccountName)
    {

    }

    public void OnFailToReRegister(String sAccountName, int nStatusCode, String sReasonPhrase)
    {
        PostStatusMsg("Re-Register: " + sReasonPhrase);
    }

    public void OnSuccessToReRegister(String sAccountName)
    {

    }

    /////////////////////////////////////////////////////////////////////////////////////////
    /////////////////////////////////////////////////////////////////////////////////////////

    public void OnTryingToUnRegister(String sAccountName)
    {
        PostStatusMsg("Un-Registering Account");
    }

    public void OnFailToUnRegister(String sAccountName)
    {
        PostStatusMsg("Un-Register: Failed");
    }

    public void OnSuccessToUnRegister(String sAccountName)
    {
        PostStatusMsg("Account Un-Registered");
    }

    /////////////////////////////////////////////////////////////////////////////////////////
    /////////////////////////////////////////////////////////////////////////////////////////

    public void OnDialCallStarted(String sAccountName, int nLineNo, String sCallerName, String sCallerId, String sDialNo)
    {
        PostStatusMsg("Call: Connecting");
    }

    public void OnDialingCall(String sAccountName, int nLineNo, int nStatusCode, String sReasonPhrase)
    {
        PostStatusMsg("Call: Dialing");
    }

    public void OnDialCallFailed(String sAccountName, int nLineNo, int nStatusCode, String sReasonPhrase, String sContact)
    {
        PostStatusMsg("Call: " + sReasonPhrase);
    }

    //////////////////////////////////////////////////////////////////////////////////////////////
    //////////////////////////////////////////////////////////////////////////////////////////////

    public void OnConnectedCall(String sAccountName, int nLineNo, String sToRTPIP, int nToRTPPort)
    {
        PostStatusMsg("Call: Connected");
    }

    public void OnDisconnectedCall(int nLineNo)
    {
        PostStatusMsg("Call: Disconnected");
    }

    public void OnHungupCall(String sAccountName, int nLineNo)
    {
        PostStatusMsg("Call: Hungup");
    }

    /////////////////////////////////////////////////////////////////////////////////////////
    /////////////////////////////////////////////////////////////////////////////////////////

    public void OnTransferCallAccepted(String sAccountName, int nLineNo)
    {
        PostStatusMsg("Transfer: Accepted");
    }

    public void OnTransferCallFailed(String sAccountName, int nLineNo, int nStatusCode, String sReasonPhrase)
    {
        PostStatusMsg("Transfer: " + sReasonPhrase);
    }

    /////////////////////////////////////////////////////////////////////////////////////////////
    /////////////////////////////////////////////////////////////////////////////////////////////

    public void OnSuccessToHold(String sAccountName, int nLineNo)
    {
        PostStatusMsg("Hold: Successful");
    }

    public void OnTryingToHold(String sAccountName, int nLineNo)
    {
        PostStatusMsg("Hold: Trying");
    }

    public void OnFailToHold(String sAccountName, int nLineNo)
    {
        PostStatusMsg("Hold: Failed");
    }

    public void OnSuccessToUnHold(String sAccountName, int nLineNo)
    {
        PostStatusMsg("UnHold: Successful");
    }

    public void OnTryingToUnHold(String sAccountName, int nLineNo)
    {
        PostStatusMsg("UnHold: Trying");
    }

    public void OnFailToUnHold(String sAccountName, int nLineNo)
    {
        PostStatusMsg("UnHold: Failed");
    }

    /////////////////////////////////////////////////////////////////////////////////////////////
    /////////////////////////////////////////////////////////////////////////////////////////////

    public void OnNetworkReachability(boolean bAvailable)
    {
        String sMsgText;

        if (bAvailable)
        {
            sMsgText = "Network is available";
        }
        else
        {
            sMsgText = "Network is not available";
        }

        PostStatusMsg(sMsgText);
    }

    /////////////////////////////////////////////////////////////////////////////////////////////
    /////////////////////////////////////////////////////////////////////////////////////////////

    public void OnVaxErrorMsg(String sFuncName, String sErrorMsg, int nErrorCode)
    {
        if (sFuncName.equals("OpenVideoDev"))
        {
            if (m_bVideoCaptureErrorShown)
                return;

            m_bVideoCaptureErrorShown = true;
        }

        PostStatusMsg(sErrorMsg);
    }

    /////////////////////////////////////////////////////////////////////////////////////////////
    /////////////////////////////////////////////////////////////////////////////////////////////

    public void OnTryingToRefresh(String sAccountName, int nLineNo)
    {

    }
    public void OnFailToRefresh(String sAccountName, int nLineNo)
    {

    }
    public void OnSuccessToRefresh(String sAccountName, int nLineNo)
    {

    }

    /////////////////////////////////////////////////////////////////////////////////////////////
    /////////////////////////////////////////////////////////////////////////////////////////////

}
