package com.vaxsoft.vaxphonesip.VaxPhoneSIP.VaxPhoneAll;

import com.vaxsoft.vaxphonesip.VaxPhoneUI.MainDrawer.MainDrawerSettings.VideoBitrateDialog;
import com.vaxsoft.vaxphonesip.VaxPhoneUI.MainDrawer.MainDrawerSettings.VideoCodecsDialog;
import com.vaxsoft.vaxphonesip.VaxPhoneUI.MainDrawer.MainDrawerSettings.VideoQualityDialog;

import com.vaxsoft.vaxphonesip.VaxPhoneUI.MainTab.CallTab.CallTabFragment;
import com.vaxsoft.vaxphonesip.VaxUserAgentSIP.IVaxUserAgentSIP;
import com.vaxsoft.vaxphonesip.VaxUserAgentSIP.VaxUserAgentSIP;

public class VaxPhoneVideo implements IVaxUserAgentSIP
{
    private static VaxPhoneVideo mSelf = null;
    private VaxUserAgentSIP mVaxAgentSIP = null;

    private static boolean m_bVideoEnabled = true;
    private boolean m_bVideoDeviceCaptured = false;

    private boolean m_bFrontVideoCameraCaptured = true;
    private boolean m_bVideoStreamStarted = false;

    ////////////////////////////////////////////////////////////////////////////////
    ////////////////////////////////////////////////////////////////////////////////

    public VaxPhoneVideo()
    {
        mSelf = this;
        mVaxAgentSIP = new VaxUserAgentSIP(this);
    }

    static public VaxPhoneVideo GetSelf()
    {
        return VaxPhoneVideo.mSelf;
    }

    ///////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////

    public boolean IsVideoStreamStarted()
    {
        return m_bVideoStreamStarted;
    }

    public static boolean IsVideoEnabled()
    {
        return m_bVideoEnabled;
    }

    public boolean EnableVideo(boolean bEnable)
    {
        if(!mVaxAgentSIP.EnableVideo(0, bEnable, bEnable))
        {
            return false;
        }

        m_bVideoEnabled = bEnable;
        return true;
    }

    ///////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////

    public void ApplyVideoQuality()
    {
        if(!m_bVideoDeviceCaptured)
            return;

        CloseVideoDevice();
        OpenVideoDevice();
    }

    ///////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////

    public boolean OpenVideoDevice()
    {
        if(m_bVideoDeviceCaptured == true)
            return true;

        CloseVideoDevice();

        int nQuality = VideoQualityDialog.GetVideoQuality();

        int nDeviceId = 0;

        if(m_bFrontVideoCameraCaptured)
            nDeviceId = 1;

        if(!mVaxAgentSIP.OpenVideoDev(nDeviceId, nQuality))
        {
            return false;
        }

        m_bVideoDeviceCaptured = true;
        return true;
    }

    public void CloseVideoDevice()
    {
        m_bVideoDeviceCaptured = false;
        mVaxAgentSIP.CloseVideoDev();
    }

    ///////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////

    public void RotateVideoDevice()
    {
        if(!m_bVideoDeviceCaptured)
            return;

        m_bFrontVideoCameraCaptured = !m_bFrontVideoCameraCaptured;

        CloseVideoDevice();
        OpenVideoDevice();
    }

    public void ApplyVideoBitrate(int nQuality)
    {
        mVaxAgentSIP.VideoCodecBitRate(VaxUserAgentSIP.VAX_VIDEO_CODEC_VP8, nQuality);
    }

    //////////////////////////////////////////////////////////////////////////////////////////
    //////////////////////////////////////////////////////////////////////////////////////////

    public void OnInitialized()
    {
        VideoBitrateDialog.PostInitialized();
        VideoCodecsDialog.PostInitialize();
    }

    public void OnUnInitialized()
    {
        CloseVideoDevice();
    }

    /////////////////////////////////////////////////////////////////////////////////////////////
    /////////////////////////////////////////////////////////////////////////////////////////////

    public void OnVideoDeviceFrameRGB(int nDeviceId, byte[] pFrameRGB, int nFrameSize, int nFrameWidth, int nFrameHeight)
    {
        CallTabFragment.PostVideoDeviceFrameRGB(nDeviceId, pFrameRGB, nFrameSize, nFrameWidth, nFrameHeight);
    }

    public void OnVideoRemoteFrameRGB(String sAccountName, int nLineNo, byte[] pFrameRGB, int nFrameSize, int nFrameWidth, int nFrameHeight)
    {
        CallTabFragment.PostOnVideoRemoteFrameRGB(nLineNo, pFrameRGB, nFrameSize, nFrameWidth, nFrameHeight);
    }

    public void OnVideoRemoteStarted(String sAccountName, int nLineNo)
    {
        m_bVideoStreamStarted = true;
        CallTabFragment.OnVideoRemoteStarted(nLineNo);
    }

    public void OnVideoRemoteEnded(String sAccountName, int nLineNo)
    {
        m_bVideoStreamStarted = false;
        CallTabFragment.OnVideoRemoteEnded(nLineNo);
    }


}
