package com.vaxsoft.vaxphonesip.VaxPhoneSIP;

import android.util.Log;

import com.vaxsoft.vaxphonesip.VaxMainUTIL.BackForeNotify.IVaxBackForeNotify;
import com.vaxsoft.vaxphonesip.VaxMainUTIL.BackForeNotify.VaxBackForeNotify;
import com.vaxsoft.vaxphonesip.VaxPhoneUI.AccountLogin.AccountLoginActivity;
import com.vaxsoft.vaxphonesip.VaxPhoneUI.IncomingCall.IncomingCallActivity;
import com.vaxsoft.vaxphonesip.VaxPhoneUI.MainDrawer.MainDrawerSettings.NetworkFragment;

import com.vaxsoft.vaxphonesip.VaxPhoneSIP.VaxConnectionService.VaxConnectionServiceEx;
import com.vaxsoft.vaxphonesip.VaxPhoneSIP.VaxPhoneAll.VaxPhoneChat;
import com.vaxsoft.vaxphonesip.VaxPhoneSIP.VaxPhoneAll.VaxPhoneCodec;
import com.vaxsoft.vaxphonesip.VaxPhoneSIP.VaxPhoneAll.VaxPhoneDTMF.VaxPhoneDTMF;
import com.vaxsoft.vaxphonesip.VaxPhoneSIP.VaxPhoneAll.VaxPhoneGeneral;
import com.vaxsoft.vaxphonesip.VaxPhoneSIP.VaxPhoneAll.VaxPhoneLog;
import com.vaxsoft.vaxphonesip.VaxPhoneSIP.VaxPhoneAll.VaxPhoneMicSpk;

import com.vaxsoft.vaxphonesip.VaxPhoneSIP.VaxPhoneAll.VaxPhoneRing.VaxPhoneRing;
import com.vaxsoft.vaxphonesip.VaxPhoneSIP.VaxPhoneAll.VaxPhoneStatus;
import com.vaxsoft.vaxphonesip.VaxPhoneSIP.VaxPhoneAll.VaxPhoneVideo;

import com.vaxsoft.vaxphonesip.VaxPhoneSIP.VaxPhoneAll.VaxPhoneVoiceChanger;
import com.vaxsoft.vaxphonesip.VaxStoreData.StoreAccount;

import com.vaxsoft.vaxphonesip.VaxUserAgentSIP.IVaxUserAgentSIP;
import com.vaxsoft.vaxphonesip.VaxUserAgentSIP.VaxUserAgentSIP;

public class VaxPhoneSIP implements IVaxUserAgentSIP, IVaxBackForeNotify
{
    public static final String ACCOUNT_VAXVOIP = "VaxAccountSIP";
    public static final String PUSH_BUNDLE_ID = "com.vaxsoft.vaxphonesip";

    private static final int ACCOUNT_REGISTER_EXPIRE = 1800;

    private static VaxPhoneSIP mSelf = null;
    private VaxUserAgentSIP mVaxAgentSIP = null;

    private VaxPhoneChat mVaxPhoneChat = null;
    private VaxPhoneVideo mVaxPhoneVideo = null;

    private VaxPhoneLog mVaxPhoneLog = null;
    private VaxPhoneStatus mVaxPhoneStatus = null;

    private VaxPhoneMicSpk mVaxPhoneMicSpk = null;
    private VaxPhoneGeneral mVaxPhoneGeneral = null;
    private VaxPhoneVoiceChanger mVaxPhoneVC = null;

    private VaxPhoneRing mVaxPhoneRing = null;
    private VaxPhoneCodec mVaxPhoneCodec = null;
    private VaxPhoneDTMF mVaxPhoneDTMF = null;

    private VaxBackForeNotify mBackForeNotify = null;

    private boolean m_bOnline = false;
    private boolean m_bRegistrationSIP = false;

    ///////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////

    public VaxPhoneSIP()
    {
        mSelf = this;

        mBackForeNotify = new VaxBackForeNotify(this);

        mVaxPhoneChat = new VaxPhoneChat();
        mVaxPhoneVideo = new VaxPhoneVideo();

        mVaxPhoneLog = new VaxPhoneLog();
        mVaxPhoneStatus = new VaxPhoneStatus();

        mVaxPhoneMicSpk = new VaxPhoneMicSpk();
        mVaxPhoneGeneral = new VaxPhoneGeneral();
        mVaxPhoneVC = new VaxPhoneVoiceChanger();

        mVaxPhoneRing = new VaxPhoneRing();
        mVaxPhoneCodec = new VaxPhoneCodec();
        mVaxPhoneDTMF = new VaxPhoneDTMF();

        mVaxAgentSIP = new VaxUserAgentSIP(this);

        Log.e("VaxPhoneSIP", "+++ VaxPhoneSIP() +++");
    }

    static public VaxPhoneSIP GetSelf()
    {
        return VaxPhoneSIP.mSelf;
    }

    ////////////////////////////////////////////////////////////////////////////////////////////
    ////////////////////////////////////////////////////////////////////////////////////////////

    public boolean IsOnline()
    {
        return m_bOnline;
    }

    ////////////////////////////////////////////////////////////////////////////////////////////
    ////////////////////////////////////////////////////////////////////////////////////////////

    public boolean Initialize()
    {
        StoreAccount objStore = new StoreAccount();

        String sUserName = objStore.GetUserName();
        String sDisplayName = objStore.GetDisplayName();

        String sAuthLogin = objStore.GetAuthLogin();
        String sAuthPwd = objStore.GetAuthPwd();

        String sDomainRealm = objStore.GetDomainRealm();

        String sServerAddr = objStore.GetServerAddr();
        String sServerPort = objStore.GetServerPort();

        boolean bRegistrationSIP = objStore.GetRegistration();

        if(sAuthLogin.length() == 0)
        {
            return false;
        }

        if(sServerAddr.length() == 0)
        {
            return false;
        }

        ///////////////////////////////////////////////////////////////////////////////

        mVaxAgentSIP.SetLicenceKey(VaxUserAgentSIP.VAX_TRIAL_LICENSE_KEY);

        ///////////////////////////////////////////////////////////////////////////////

        int nListenPortSIP = -1;

        if(!NetworkFragment.IsRandomPortSIP())
            nListenPortSIP = NetworkFragment.GetNetworkPortSIP();

        ///////////////////////////////////////////////////////////////////////////////

        if(!mVaxAgentSIP.Initialize())
        {
            String sErrorMSg = mVaxAgentSIP.GetVaxErrorMsg();

            return false;
        }

        mVaxPhoneRing.ActivateRing();

        mVaxAgentSIP.RemoveAccountSIP(ACCOUNT_VAXVOIP);

        if(!mVaxAgentSIP.AddAccountSIP(ACCOUNT_VAXVOIP, VaxUserAgentSIP.VAX_ACCOUNT_TYPE_UDP, "", nListenPortSIP, sDisplayName, sUserName, sAuthLogin, sAuthPwd, sDomainRealm, sServerAddr, Integer.parseInt(sServerPort)))
            return false;

        m_bRegistrationSIP = bRegistrationSIP;

        if(bRegistrationSIP)
        {
            mVaxAgentSIP.RegisterAccountPushKit(ACCOUNT_VAXVOIP, ACCOUNT_REGISTER_EXPIRE);
        }

        mVaxAgentSIP.AutoRegistration(ACCOUNT_VAXVOIP, true, -1, 20);
        mVaxAgentSIP.EnableKeepAlive(ACCOUNT_VAXVOIP, 30); // 10 seconds keep alive;

        mBackForeNotify.BackForegroundNotify(true);
        mVaxAgentSIP.NetworkReachability(true);

        m_bOnline = true;
        return true;
    }

    /////////////////////////////////////////////////////////////////////////////////////
    /////////////////////////////////////////////////////////////////////////////////////
    public void UnInitialize()
    {
        m_bOnline = false;
        m_bRegistrationSIP = false;

        mVaxAgentSIP.NetworkReachability(false);
        mBackForeNotify.BackForegroundNotify(false);

        mVaxAgentSIP.UnRegisterAccountSIP(ACCOUNT_VAXVOIP);
    }

    ///////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////

    public boolean DialCall(String sDialNo)
    {
        return mVaxAgentSIP.DialCall(ACCOUNT_VAXVOIP, 0, "", "", sDialNo);
    }

    public boolean DisconnectCall()
    {
        return mVaxAgentSIP.DisconnectCall(0);
    }

    /////////////////////////////////////////////////////////////////////////////////////
    /////////////////////////////////////////////////////////////////////////////////////

    public void OnSuccessToRegister(String sAccountName)
    {
        AccountLoginActivity.PostRegisterSuccessAccountSIP();
    }

    public void OnVaxRegisterSuccessPushKit(String sDeviceToken)
    {
        if(!m_bRegistrationSIP)
            return;

        mVaxAgentSIP.SetPushSIP(ACCOUNT_VAXVOIP, PUSH_BUNDLE_ID, sDeviceToken);
        mVaxAgentSIP.RegisterAccountSIP(ACCOUNT_VAXVOIP, ACCOUNT_REGISTER_EXPIRE);
    }

    public void OnVaxRegisterNewTokenPushKit(String sDeviceToken)
    {
        if(!m_bRegistrationSIP)
            return;

        mVaxAgentSIP.SetPushSIP(ACCOUNT_VAXVOIP, PUSH_BUNDLE_ID, sDeviceToken);
        mVaxAgentSIP.RegisterAccountSIP(ACCOUNT_VAXVOIP, ACCOUNT_REGISTER_EXPIRE);
    }

    /////////////////////////////////////////////////////////////////////////////////////////
    /////////////////////////////////////////////////////////////////////////////////////////

    public void OnIncomingCallPushKit(String sCallerName, String sCallerId)
    {
        if(!m_bRegistrationSIP || mVaxAgentSIP.IsLineBusy(0))
        {
            return;
        }

        mVaxAgentSIP.RegisterAccountPushKit(ACCOUNT_VAXVOIP, ACCOUNT_REGISTER_EXPIRE);

        VaxConnectionServiceEx.AddNewIncomingCall(sCallerName, sCallerId);
    }

    public void OnIncomingCallStarted(String sAccountName, String sCallerName, String sCallerId)
    {
        if(mVaxAgentSIP.IsLineConnected(0))
        {
            mVaxAgentSIP.RejectCall();
            return;
        }

        VaxConnectionServiceEx.AddNewIncomingCall(sCallerName, sCallerId);
    }

    public void OnIncomingCallEnded(String sAccountName, int nReasonCode)
    {
        IncomingCallActivity.HideIncomingUI();
    }

    /////////////////////////////////////////////////////////////////////////////////////////////
    /////////////////////////////////////////////////////////////////////////////////////////////

    public void OnNetworkReachability(boolean bAvailable)
    {
        if (bAvailable == false || IsOnline() == false)
            return;

         if(mVaxAgentSIP.IsLineConnected(0))
            mVaxAgentSIP.RefreshLine(0);
    }

    ///////////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////

    public void OnApplicationEnterForeground()
    {
        if(m_bRegistrationSIP)
            mVaxAgentSIP.RegisterAccountPushKit(ACCOUNT_VAXVOIP, ACCOUNT_REGISTER_EXPIRE);
    }
}
