package com.vaxsoft.vaxphonesip.VaxPhoneUI.AccountLogin;

import android.Manifest;
import android.content.Intent;
import android.content.pm.PackageManager;
import android.os.Build;
import android.os.Handler;
import androidx.annotation.NonNull;
import androidx.core.app.ActivityCompat;
import androidx.fragment.app.FragmentManager;
import androidx.fragment.app.FragmentTransaction;
import androidx.appcompat.app.AppCompatActivity;
import android.os.Bundle;
import android.view.View;
import android.widget.Button;
import android.widget.EditText;
import android.widget.Switch;
import android.widget.TextView;

import com.vaxsoft.vaxphonesip.R;
import com.vaxsoft.vaxphonesip.VaxPhoneUI.CustomViews.DialogView;
import com.vaxsoft.vaxphonesip.VaxPhoneUI.MainTab.MainTabActivity;
import com.vaxsoft.vaxphonesip.VaxPhoneSIP.VaxPhoneAll.VaxPhoneGeneral;
import com.vaxsoft.vaxphonesip.VaxPhoneSIP.VaxPhoneSIP;
import com.vaxsoft.vaxphonesip.VaxStoreData.StoreAccount;
import com.vaxsoft.vaxphonesip.VaxUserAgentSIP.IVaxUserAgentSIP;
import com.vaxsoft.vaxphonesip.VaxUserAgentSIP.VaxUserAgentSIP;

public class AccountLoginActivity extends AppCompatActivity implements IVaxUserAgentSIP
{
    private static AccountLoginActivity mSelf = null;
    private static String msAccountLoginStatusText = "";

    private final String DEFAULT_SIP_PORT = "5060";

    private final int ID_REQUEST_READ_CONTACTS = 1000;
    private final int ID_REQUEST_RECORD_AUDIO = 2000;
    private final int ID_REQUEST_CAPTURE_CAMERA = 3000;
    private final int ID_REQUEST_PUSH_NOTIFICATION = 4000;

    EditText EditTextAuthLogin, EditTextAuthPwd, EditTextServerURI;
    Switch SwitchRegisterSIP;
    TextView TextViewStatus;
    Button BtnLogin;

    private Handler mDelayHandler = null;

    //////////////////////////////////////////////////////////////////////////////////////
    //////////////////////////////////////////////////////////////////////////////////////

    @Override
    protected void onCreate(Bundle savedInstanceState)
    {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_log_in);

        mSelf = this;
        msAccountLoginStatusText = "Account is offline";

        EditTextAuthLogin = findViewById(R.id.username);
        EditTextAuthPwd = findViewById(R.id.password);
        EditTextServerURI = findViewById(R.id.server_ip_domain);

        SwitchRegisterSIP = findViewById(R.id.sip_registration);

        BtnLogin = findViewById(R.id.BtnLogin);
        TextViewStatus = findViewById(R.id.TextViewStatus);

        DisplayAccountInfo();
    }

    public void DisplayAccountInfo()
    {
        StoreAccount objStore = new StoreAccount();

        String sAuthLogin = objStore.GetAuthLogin();
        String sAuthPwd = objStore.GetAuthPwd();

        String sServerAddr = objStore.GetServerAddr();
        String sServerPort = objStore.GetServerPort();

        boolean bRegisterSIP = objStore.GetRegistration();

        EditTextAuthLogin.setText(sAuthLogin);
        EditTextAuthPwd.setText(sAuthPwd);

        String sServerURI = "";

        if (sServerAddr.length() != 0)
        {
            if(sServerPort.length() == 0)
                sServerPort = DEFAULT_SIP_PORT;

            sServerURI = sServerAddr + ":" + sServerPort;
        }

        EditTextServerURI.setText(sServerURI);
        SwitchRegisterSIP.setChecked(bRegisterSIP);

        UpdateOnlineBtnText();

        TextViewStatus.setText(msAccountLoginStatusText);
    }

    //////////////////////////////////////////////////////////////////////////////////////
    //////////////////////////////////////////////////////////////////////////////////////

    public void BtnAccountInfo(View view)
    {
        StoreAccountInfo();

        AccountInfoFragment objAccountInfo = new AccountInfoFragment();

        FragmentManager objFragManager = getSupportFragmentManager();
        FragmentTransaction objFragTransaction = objFragManager.beginTransaction();

        objFragTransaction.add(android.R.id.content, objAccountInfo ,"Account Info");
        objFragTransaction.addToBackStack("Account Info");
        objFragTransaction.commit();
    }

    //////////////////////////////////////////////////////////////////////////////////////
    //////////////////////////////////////////////////////////////////////////////////////

    public void BtnLogIn(View view)
    {
        StoreAccountInfo();

        if(VaxPhoneSIP.GetSelf().IsOnline())
        {
            AccountOffline();
            return;
        }

        AccountOnline();
    }

    //////////////////////////////////////////////////////////////////////////////////////
    //////////////////////////////////////////////////////////////////////////////////////

    private void AccountOffline()
    {
        ShowMainTabBar(false);
        VaxPhoneSIP.GetSelf().UnInitialize();
        UpdateOnlineBtnText();

        StoreAccount.SetAccountStatusOnline(false);
    }

    private void AccountOnline()
    {
        if(!VaxPhoneGeneral.GetSelf().IsNetworkAvailable())
        {
            AccountLoginActivity.PostStatusText("Network is not available");
            return;
        }

        StoreAccountInfo();

        boolean bRegisterSIP = SwitchRegisterSIP.isChecked();

        if(!VaxPhoneSIP.GetSelf().Initialize())
            return;

        ShowTrialMsg();

        UpdateOnlineBtnText();

        if(!bRegisterSIP)
        {
            StoreAccount.SetAccountStatusOnline(true);
            ShowMainTabBar(true);
        }
    }

    private void ShowTrialMsg()
    {
        if(!VaxUserAgentSIP.IsLicenceKeyTrial())
            return;

        String sTrialMsg;

        sTrialMsg = "VaxVoIP SDK is free to use for 30 days.\n\n";
        sTrialMsg = sTrialMsg + "If you want to continue using it after the trial period, you must pay the License fee.\n\n";
        sTrialMsg = sTrialMsg + "Thank you for using VaxVoIP SDK!\n\nWeb: www.vaxvoip.com\nEmail: info@vaxvoip.com";

        DialogView.ShowDialog(this, sTrialMsg);
    }

    //////////////////////////////////////////////////////////////////////////////////////
    //////////////////////////////////////////////////////////////////////////////////////

    private void UpdateOnlineBtnText()
    {
        if(VaxPhoneSIP.GetSelf().IsOnline())
            BtnLogin.setText("LOG OUT");
        else
            BtnLogin.setText("LOG IN");
    }

    //////////////////////////////////////////////////////////////////////////////////////
    //////////////////////////////////////////////////////////////////////////////////////

    public void BtnGetDemoAccount(View view)
    {
        long nDateTime = System.currentTimeMillis();
        String sDateTime = String.valueOf(nDateTime%9999999);

        EditTextAuthLogin.setText(sDateTime);
        EditTextAuthPwd.setText(sDateTime);

        EditTextServerURI.setText("demo.vaxvoip.com:8060");
        SwitchRegisterSIP.setEnabled(true);

        StoreAccount objStore = new StoreAccount();

        objStore.SetUserName("");
        objStore.SetDisplayName("");
        objStore.SetDomainRealm("");

        ShowTestAccountInfo(sDateTime, sDateTime, "demo.vaxvoip.com", "8060");
    }

    private void ShowTestAccountInfo(String sAuthLogin, String sAuthPwd, String sServerAddr, String sServerPort)
    {
        String sDemoMsg;

        sDemoMsg = "\n" + "Auth Login: " + sAuthLogin + "\n";
        sDemoMsg = sDemoMsg + "Auth Password: " + sAuthPwd + "\n";
        sDemoMsg = sDemoMsg + "Server Addr: " + sServerAddr + "\n";
        sDemoMsg = sDemoMsg + "Server Port: " + sServerPort + "\n \n";

        String sMsg = sDemoMsg + "Demo accounts are only for account-to-account VoIP calls. Use demo accounts in different devices to dial/receive VoIP calls.\n\nDial 000 for self testing.\n\n[Powered by VaxVoIP Server SDK]";

        DialogView.ShowDialog(this, sMsg);
    }

    //////////////////////////////////////////////////////////////////////////////////////
    //////////////////////////////////////////////////////////////////////////////////////

    private void StoreAccountInfo()
    {
        String sAuthLogin = EditTextAuthLogin.getText().toString();
        String sAuthPwd = EditTextAuthPwd.getText().toString();

        String sServerURI = EditTextServerURI.getText().toString();
        boolean bRegisterSIP = SwitchRegisterSIP.isChecked();

        String sServerAddr = sServerURI;
        String sServerPort = DEFAULT_SIP_PORT;

        if (sServerURI.contains(":"))
        {
            String aServerAddr[] = sServerURI.split(":");

            sServerAddr = aServerAddr[0];
            sServerPort = aServerAddr[aServerAddr.length - 1];
        }

        StoreAccount objStore = new StoreAccount();

        String sReadA = objStore.GetAuthLogin();
        String sReadB = objStore.GetServerAddr();

        if(sAuthLogin.compareToIgnoreCase(objStore.GetAuthLogin()) != 0 || sServerAddr.compareToIgnoreCase(objStore.GetServerAddr()) != 0)
        {
            objStore.SetUserName("");
            objStore.SetDisplayName("");
            objStore.SetDomainRealm("");
        }

        objStore.SetAuthLogin(sAuthLogin);
        objStore.SetAuthPwd(sAuthPwd);

        objStore.SetServerAddr(sServerAddr);
        objStore.SetServerPort(sServerPort);

        objStore.SetRegistration(bRegisterSIP);
    }

    //////////////////////////////////////////////////////////////////////////////////////
    //////////////////////////////////////////////////////////////////////////////////////

    void ShowMainTabBar(boolean bStart)
    {
        if (mDelayHandler != null)
        {
            mDelayHandler.removeCallbacksAndMessages(null);
            mDelayHandler = null;
        }

        if(bStart == false)
            return;

        mDelayHandler = new Handler();

        mDelayHandler.postDelayed(new Runnable()
        {
            @Override
            public void run()
            {
                Intent objIntent = new Intent(AccountLoginActivity.this, MainTabActivity.class);
                startActivity(objIntent);

                mDelayHandler = null;
                finish();
            }
        }, 1000);

    }

    //////////////////////////////////////////////////////////////////////////////////////
    //////////////////////////////////////////////////////////////////////////////////////

    private void ShowStatusMsg(String sMsg)
    {
        TextViewStatus.setText(sMsg);
    }

    ///////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////

    static public void PostStatusText(String sMsg)
    {
        msAccountLoginStatusText = sMsg;

        if (mSelf == null)
            return;

        mSelf.ShowStatusMsg(sMsg);
    }

    static public void PostRegisterSuccessAccountSIP()
    {
        if (mSelf == null)
            return;

        StoreAccount.SetAccountStatusOnline(true);
        mSelf.ShowMainTabBar(true);
    }

    static public void PostInitialized()
    {
        if (mSelf == null)
            return;


    }

    ///////////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////
    
    private void AskPermission()
    {
        if (ActivityCompat.checkSelfPermission(this, Manifest.permission.READ_CONTACTS) != PackageManager.PERMISSION_GRANTED)
        {
            ActivityCompat.requestPermissions(this, new String[]{Manifest.permission.READ_CONTACTS}, ID_REQUEST_READ_CONTACTS);
        }

        if (ActivityCompat.checkSelfPermission(this, Manifest.permission.RECORD_AUDIO) != PackageManager.PERMISSION_GRANTED)
        {
            ActivityCompat.requestPermissions(this, new String[]{Manifest.permission.RECORD_AUDIO}, ID_REQUEST_RECORD_AUDIO);
        }

        if (ActivityCompat.checkSelfPermission(this, Manifest.permission.CAMERA) != PackageManager.PERMISSION_GRANTED)
        {
            requestPermissions(new String[]{Manifest.permission.CAMERA}, ID_REQUEST_CAPTURE_CAMERA);
        }

        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.TIRAMISU)
        {
            if (ActivityCompat.checkSelfPermission(this, Manifest.permission.POST_NOTIFICATIONS) != PackageManager.PERMISSION_GRANTED)
            {
                requestPermissions(new String[]{Manifest.permission.POST_NOTIFICATIONS}, ID_REQUEST_PUSH_NOTIFICATION);
            }
        }
    }

    @Override
    public void onRequestPermissionsResult(int nRequestCode, @NonNull String[] sPermissions, @NonNull int[] aGrantResults)
    {
        if (nRequestCode == ID_REQUEST_READ_CONTACTS)
        {
            /*
            if (aGrantResults.length > 0 && aGrantResults[0] == PackageManager.PERMISSION_DENIED)
            {
                boolean bResult = ActivityCompat.shouldShowRequestPermissionRationale(this, Manifest.permission.READ_CONTACTS);

                if (bResult)
                    DialogUtil.ShowDialog(this, "Please allow VaxPhone to read contacts so you can use full features of app.");

                if (!bResult)
                    DialogUtil.ShowDialog(this, "Please Allow Manually");
            }
            */
        }
    }

    @Override
    protected void onResume()
    {
        super.onResume();
        AskPermission();
    }

    @Override
    protected void onDestroy()
    {
        mSelf = null;
        super.onDestroy();
    }
}
