package com.vaxsoft.vaxphonesip.VaxPhoneUI.MainNotify;

import android.app.Notification;
import android.app.NotificationChannel;
import android.app.NotificationManager;
import android.app.PendingIntent;
import android.content.Context;
import android.content.ContextWrapper;
import android.content.Intent;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.os.Build;
import androidx.core.app.NotificationCompat;

import com.vaxsoft.vaxphonesip.VaxMainAPP.VaxPhoneAPP;
import com.vaxsoft.vaxphonesip.VaxPhoneUI.MainTab.ChatTab.ChatContactTabFragment;
import com.vaxsoft.vaxphonesip.R;

public class VaxPhoneNotify extends ContextWrapper
{
    private Context mContext;

    private NotificationManager mNotificationManager = null;

    private static final String PRIMARY_CHANNEL = "default";

    private static final String APP_ICON = "AppIcon";
    private static final String MSG_NOTIFICATION = "Message";

    private static final int MSG_NOTIFICATION_ID = 1001;
    private static final int CHAT_FRAGMENT_TAB_INDEX = 2;

    //////////////////////////////////////////////////////////////////////////////////////////
    //////////////////////////////////////////////////////////////////////////////////////////

    public VaxPhoneNotify()
    {
        super(VaxPhoneAPP.GetContextAPP());

        mContext = VaxPhoneAPP.GetContextAPP();
    }

    //////////////////////////////////////////////////////////////////////////////////////////////
    //////////////////////////////////////////////////////////////////////////////////////////////

    private PendingIntent CreatePendingIntentActivity(Class activity, int nSelectedTab)
    {
        Intent objIntent = new Intent(mContext, activity);
        objIntent.putExtra("SelectedTab", nSelectedTab);

        return PendingIntent.getActivity(mContext, 0, objIntent, PendingIntent.FLAG_CANCEL_CURRENT);
    }

    //////////////////////////////////////////////////////////////////////////////////////////////
    //////////////////////////////////////////////////////////////////////////////////////////////

    public void StartMsgNotification(String sUserName, String sMsgText, Class objClass)
    {
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.O)
        {
            NotificationChannel objNotificationChannel = new NotificationChannel(PRIMARY_CHANNEL, "Default-Notifications", NotificationManager.IMPORTANCE_DEFAULT);
            objNotificationChannel.setDescription("VaxPhone Default Notifications");

            NotificationManager objNotificationManager = GetNotificationManager();

            if (objNotificationManager != null)
                objNotificationManager.createNotificationChannel(objNotificationChannel);
        }

        PendingIntent objPendingIntent = CreatePendingIntentActivity(objClass, CHAT_FRAGMENT_TAB_INDEX);

        int nMissedMsgCount = ChatContactTabFragment.ReadMissedCount() + 1;

        String sTitle = "Chat Messages ";

        String sMsg = "You have " + nMissedMsgCount + " new";
        String sContentText = nMissedMsgCount > 1 ? sMsg + " messages." : sMsg + " message.";

        Notification objNotification = CreateNotification(sTitle, sContentText, objPendingIntent);

        ShowNotification(MSG_NOTIFICATION_ID, objNotification);
    }

    //////////////////////////////////////////////////////////////////////////////////////////////
    //////////////////////////////////////////////////////////////////////////////////////////////

    private Notification CreateNotification(String sContentTitle, String sContentText, PendingIntent sContentIntent)
    {
        NotificationCompat.Builder objNotificationBuilder = new NotificationCompat.Builder(this, PRIMARY_CHANNEL);

        if (sContentTitle != null)
            objNotificationBuilder.setContentTitle(sContentTitle);

        if (sContentText != null)
            objNotificationBuilder.setContentText(sContentText);

        objNotificationBuilder.setSmallIcon(GetSmallIcon(MSG_NOTIFICATION));
        objNotificationBuilder.setLargeIcon(GetLargeIcon(MSG_NOTIFICATION));

        if (sContentIntent != null)
            objNotificationBuilder.setContentIntent(sContentIntent);

        objNotificationBuilder.setPriority(NotificationManager.IMPORTANCE_HIGH);

        objNotificationBuilder.setAutoCancel(true);
        objNotificationBuilder.setUsesChronometer(false);
        objNotificationBuilder.setShowWhen(false);
        objNotificationBuilder.setOngoing(false);
        objNotificationBuilder.setDefaults(Notification.DEFAULT_ALL);
        objNotificationBuilder.setChannelId(PRIMARY_CHANNEL);

        return objNotificationBuilder.build();
    }

    private void ShowNotification(int nNotificationId, Notification objNotification)
    {
        NotificationManager objNotificationManager = GetNotificationManager();

        if (objNotificationManager != null)
            objNotificationManager.notify(nNotificationId, objNotification);
    }

    //////////////////////////////////////////////////////////////////////////////////////////////
    //////////////////////////////////////////////////////////////////////////////////////////////

    private NotificationManager GetNotificationManager()
    {
        if (mNotificationManager == null)
            mNotificationManager = (NotificationManager) getSystemService(Context.NOTIFICATION_SERVICE);

        return mNotificationManager;
    }

    //////////////////////////////////////////////////////////////////////////////////////////////
    //////////////////////////////////////////////////////////////////////////////////////////////

    private int GetSmallIcon(String sAction)
    {
        int nIcon = -1;

        switch (sAction)
        {
            case MSG_NOTIFICATION:
                nIcon = R.drawable.ic_chat;
                break;
        }

        return nIcon;
    }

    private Bitmap GetLargeIcon(String sAction)
    {
        Bitmap bitmap = null;

        switch (sAction)
        {
            case APP_ICON:
                bitmap =  BitmapFactory.decodeResource(getResources(), R.drawable.app_icon);
                break;

            case MSG_NOTIFICATION:
                bitmap = BitmapFactory.decodeResource(getResources(), R.mipmap.ic_launcher_round);
                break;
        }

        return bitmap;
    }


    //////////////////////////////////////////////////////////////////////////////////////////////
    //////////////////////////////////////////////////////////////////////////////////////////////
}
