package com.vaxsoft.vaxphonesip.VaxPhoneUI.MainTab.CallTab;

import android.app.Activity;
import androidx.fragment.app.FragmentManager;
import androidx.fragment.app.FragmentTransaction;
import androidx.appcompat.widget.AppCompatImageView;
import android.view.View;
import android.view.animation.TranslateAnimation;
import android.widget.LinearLayout;
import android.widget.TextView;

import java.util.Timer;
import java.util.TimerTask;

import com.vaxsoft.vaxphonesip.R;
import com.vaxsoft.vaxphonesip.VaxPhoneUI.MainTab.MainTabActivity;
import com.vaxsoft.vaxphonesip.VaxPhoneSIP.VaxPhoneAll.VaxPhoneGeneral;
import com.vaxsoft.vaxphonesip.VaxPhoneSIP.VaxPhoneAll.VaxPhoneMicSpk;
import com.vaxsoft.vaxphonesip.VaxPhoneSIP.VaxPhoneSIP;
import com.vaxsoft.vaxphonesip.VaxPhoneSIP.VaxPhoneAll.VaxPhoneVideo;
import com.vaxsoft.vaxphonesip.VaxPhoneSIP.VaxPhoneAll.VaxPhoneVoiceChanger;

public class CallIconsFragment extends DialpadFragment implements View.OnClickListener
{
    private LinearLayout BtnSwitchCamera, BtnSpkPhone, BtnDialpad, BtnEndCall, BtnHoldCall;

    private Activity mActivity = null;
    private View SideIconsLayout = null;
    private AppCompatImageView BtnVideo, BtnMic, BtnMuteSpk, BtnVoiceChanger;

    private TextView mTextViewExpiry;
    private Boolean m_bShowFirstTime = true;

    private Timer m_objHideIconTick = null;
    private final int HIDE_ICON_DELAY = 8500;  // milliseconds

    CallIconsFragment(Activity activity, View view)
    {
        mActivity = activity;

        LoadViewAll(view);
        ActivateClickListeners();

        UpdateBtnViewAll(true);
    }

    ///////////////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////

    private void LoadViewAll(View view)
    {
        BtnVideo          = view.findViewById(R.id.videocam);
        BtnMic            = view.findViewById(R.id.icon_mic);
        BtnVoiceChanger   = view.findViewById(R.id.icon_voice_changer);
        BtnMuteSpk        = view.findViewById(R.id.icon_mute_spk);
        BtnSpkPhone       = view.findViewById(R.id.layout_spk_phone);
        BtnDialpad        = view.findViewById(R.id.layout_dialpad);
        BtnSwitchCamera   = view.findViewById(R.id.layout_switch_camera);
        BtnEndCall        = view.findViewById(R.id.layout_end_call);
        BtnHoldCall       = view.findViewById(R.id.layout_hold_call);

        SideIconsLayout   = view.findViewById(R.id.IconsLayout);
        mTextViewExpiry   = view.findViewById(R.id.textViewExpiry);
    }

    ///////////////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////

    private void PostHideBtnAll()
    {
        MainTabActivity objActivity = (MainTabActivity) mActivity;

        if (objActivity == null)
            return;

        objActivity.runOnUiThread(new Runnable()
        {
            @Override
            public void run()
            {
                UpdateBtnViewAll(false);
            }
        });
    }

    private void StartHideBtnAllTick()
    {
        if(m_objHideIconTick != null)
            m_objHideIconTick.cancel();

        m_objHideIconTick = new Timer();
        m_objHideIconTick.schedule(new TimerTask()
        {
            @Override
            public void run()
            {
                PostHideBtnAll();
            }
        }, HIDE_ICON_DELAY);
    }

    ///////////////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////

    private void ActivateClickListeners()
    {
        BtnSwitchCamera.setOnClickListener(this);
        BtnVideo.setOnClickListener(this);

        BtnMuteSpk.setOnClickListener(this);
        BtnSpkPhone.setOnClickListener(this);
        BtnMic.setOnClickListener(this);

        BtnVoiceChanger.setOnClickListener(this);
        BtnDialpad.setOnClickListener(this);

        BtnEndCall.setOnClickListener(this);
        BtnHoldCall.setOnClickListener(this);
    }

    ///////////////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////

    public void DisplayBtnAll()
    {
        if(SideIconsLayout.isShown() && m_bShowFirstTime == false)
        {
            UpdateBtnViewAll(false);
            return;
        }

        m_bShowFirstTime = false;
        UpdateBtnViewAll(true);
    }

    private void UpdateBtnViewAll(Boolean bShow)
    {
        StartHideBtnAllTick();

        UpdateOtherBtnView(bShow);
        UpdateEndCallBtnView(bShow);

        UpdateHoldCallBtnView(bShow);

        mTextViewExpiry.setVisibility(View.INVISIBLE);
    }

    ////////////////////////////////////////////////////////////////////////////////
    ////////////////////////////////////////////////////////////////////////////////

    private void UpdateOtherBtnView(Boolean bShow)
    {
        if(bShow)
        {
            BtnVideo.setSelected(VaxPhoneVideo.GetSelf().IsVideoEnabled());
            BtnVoiceChanger.setSelected(VaxPhoneVoiceChanger.IsVoiceChangerEnabled());

            BtnMic.setSelected(VaxPhoneMicSpk.IsMuteMic());
            BtnMuteSpk.setSelected(VaxPhoneMicSpk.IsMuteSpk());
            BtnSpkPhone.setSelected(VaxPhoneGeneral.GetSelf().IsSpeakerPhone());
        }

        int nVisibility;

        TranslateAnimation animate = null;

        if(bShow)
        {
            if(SideIconsLayout.isShown())
                return;

            nVisibility = View.VISIBLE;
            SideIconsLayout.setVisibility(nVisibility);
        }
        else
        {
            if(!SideIconsLayout.isShown())
                return;

            nVisibility = View.GONE;

            animate = new TranslateAnimation(0, SideIconsLayout.getWidth(), 0, 0);
            animate.setFillAfter(false);

            animate.setDuration(250);
            SideIconsLayout.startAnimation(animate);
            SideIconsLayout.setVisibility(nVisibility);
        }
    }

    ///////////////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////

    private void UpdateEndCallBtnView(Boolean bShow)
    {
        if (!VaxPhoneGeneral.GetSelf().IsLineBusy())
        {
            bShow = false;
        }

        int nVisibility;
        TranslateAnimation animate = null;

        if(bShow)
        {
            nVisibility = View.VISIBLE;
            BtnEndCall.setVisibility(nVisibility);

            return;
        }

        if(BtnEndCall.isShown())
        {
            nVisibility = View.GONE;
            animate = new TranslateAnimation(0, 0, 0, BtnEndCall.getWidth());

            animate.setDuration(250);
            animate.setFillAfter(false);

            BtnEndCall.startAnimation(animate);
            BtnEndCall.setVisibility(nVisibility);
        }
    }

    ///////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////

    private void UpdateHoldCallBtnView(Boolean bShow)
    {
        if (!VaxPhoneGeneral.GetSelf().IsLineBusy())
        {
            bShow = false;
        }

        ////////////////////////////////////////////////////////////

        boolean bIsLineHold = VaxPhoneGeneral.GetSelf().IsLineHold();
        BtnHoldCall.setSelected(bIsLineHold);

        ////////////////////////////////////////////////////////////

        int nVisibility;

        if(bShow)
        {
            nVisibility = View.VISIBLE;
        }
        else
        {
            nVisibility = View.INVISIBLE;
        }

        BtnHoldCall.setVisibility(nVisibility);
    }

    ///////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////

    @Override
    public void onClick(View view)
    {
        int nId = view.getId();

        if(nId == R.id.layout_switch_camera)
        {
            OnClickBtnSwitchCamera();
        }
        else if(nId == R.id.videocam)
        {
            OnClickBtnVideo();
        }
        else if(nId == R.id.icon_mic)
        {
            OnClickBtnMic();
        }
        else if(nId == R.id.icon_voice_changer)
        {
            OnClickBtnVoiceChanger();
        }
        else if(nId == R.id.layout_spk_phone)
        {
            OnClickSpkPhone();
        }
        else if(nId == R.id.icon_mute_spk)
        {
            OnClickBtnMuteSpk();
        }
        else if(nId == R.id.layout_dialpad)
        {
            OnClickBtnDialpad();
        }
        else if(nId == R.id.layout_end_call)
        {
            OnClickBtnEndCall();
        }
        else if(nId == R.id.layout_hold_call)
        {
            OnClickHoldCall();
        }

        UpdateBtnViewAll(true);
    }

    ///////////////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////

    private void OnClickBtnSwitchCamera()
    {
        VaxPhoneVideo.GetSelf().RotateVideoDevice();
    }

    private void OnClickBtnVideo()
    {
        boolean bVideoEnable = VaxPhoneVideo.GetSelf().IsVideoEnabled();
        VaxPhoneVideo.GetSelf().EnableVideo(!bVideoEnable);
    }

    private void OnClickBtnMic()
    {
        boolean bMuteMic = VaxPhoneMicSpk.GetSelf().IsMuteMic();
        VaxPhoneMicSpk.GetSelf().MuteMic(!bMuteMic);
    }

    private void OnClickBtnVoiceChanger()
    {
        boolean bIsEnabled = VaxPhoneVoiceChanger.GetSelf().IsVoiceChangerEnabled();
        VaxPhoneVoiceChanger.GetSelf().VoiceChanger(!bIsEnabled);

        BtnVoiceChanger.setSelected(!bIsEnabled);
    }

    private void OnClickBtnMuteSpk()
    {
        boolean bMuteSpk = VaxPhoneMicSpk.GetSelf().IsMuteSpk();
        VaxPhoneMicSpk.GetSelf().MuteSpk(!bMuteSpk);
    }

    private void OnClickSpkPhone()
    {
        boolean bSpkPhone = VaxPhoneGeneral.GetSelf().IsSpeakerPhone();
        VaxPhoneGeneral.GetSelf().SpeakerPhone(!bSpkPhone);
    }

    private void OnClickBtnDialpad()
    {
        FragmentManager objFragManager = ((MainTabActivity) mActivity).getSupportFragmentManager();
        FragmentTransaction objFragTransaction = objFragManager.beginTransaction();

        objFragTransaction.add(android.R.id.content, (DialpadFragment) this, "Dialpad");
        objFragTransaction.addToBackStack("Dialpad");
        objFragTransaction.commit();
    }

    public void OnDialpadClosed()
    {
        UpdateBtnViewAll(true);
    }

    private void OnClickBtnEndCall()
    {
        VaxPhoneSIP.GetSelf().DisconnectCall();
    }

    private void OnClickHoldCall()
    {
        boolean bHoldLine = VaxPhoneGeneral.GetSelf().IsLineHold();

        if (bHoldLine)
        {
            VaxPhoneGeneral.GetSelf().UnHoldLine();
            return;
        }

        VaxPhoneGeneral.GetSelf().HoldLine();
    }

    ///////////////////////////////////////////////////////////////////////////////////////////////
    //////////////////////////////////////////////////////////////////////////////////////////////
}
