package com.vaxsoft.vaxphonesip.VaxPhoneUI.MainTab.RecentTab;

import android.os.Bundle;
import androidx.annotation.Nullable;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.TextView;

import com.vaxsoft.vaxphonesip.VaxPhoneUI.CustomViews.TabFragment.CustomTabFragment;
import com.vaxsoft.vaxphonesip.R;
import com.vaxsoft.vaxphonesip.VaxPhoneUI.MainTab.CallTab.DialpadFragment;
import com.vaxsoft.vaxphonesip.VaxPhoneUI.MainTab.MainTabActivity;
import com.vaxsoft.vaxphonesip.VaxStoreData.DataPRF.DataPreference;
import com.vaxsoft.vaxphonesip.VaxStoreData.StoreRecent;

public class RecentTabFragment extends CustomTabFragment
{
    TextView TextViewNoRecords;
    RecentRecyclerView mRecentRecyclerView;

    private static RecentTabFragment mRecentFragment;
    private static int m_nRecentMissedCount = 0;

    private static final String RECENT_MISSED_COUNT_KEY = "Recent Missed Count";

    ////////////////////////////////////////////////////////////////////////////////////

    public RecentTabFragment()
    {

    }

    /////////////////////////////////////////////////////////////////////////////////
    /////////////////////////////////////////////////////////////////////////////////

    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container, Bundle savedInstanceState)
    {
        return inflater.inflate(R.layout.fragment_recent, container, false);
    }

    @Override
    public void onViewCreated(View view, @Nullable Bundle savedInstanceState)
    {
        InitViews(view);
        UpdateUI();

        super.onViewCreated(view, savedInstanceState);
    }

    private void InitViews(View view)
    {
        TextViewNoRecords = view.findViewById(R.id.no_records);
        mRecentRecyclerView = view.findViewById(R.id.recent_recyclerview);
    }

    ///////////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////

    public void OnFragmentActivated()
    {
        mRecentFragment = this;

        mRecentRecyclerView.OnFragmentActivated();
        UpdateUI();

        super.OnFragmentActivated();
    }

    public void OnFragmentDeactivated()
    {
        mRecentFragment = null;

        if (mRecentRecyclerView != null)
            mRecentRecyclerView.OnFragmentDeactivated();

        super.OnFragmentDeactivated();
    }

    //////////////////////////////////////////////////////////////////////////////////////////
    //////////////////////////////////////////////////////////////////////////////////////////

    private void UpdateUI()
    {
        StoreRecent objStore = new StoreRecent();
        int nCount = objStore.GetCallCount();

        if (nCount <= 0)
        {
            TextViewNoRecords.setVisibility(View.VISIBLE);
            mRecentRecyclerView.setVisibility(View.GONE);
        }
        else
        {
            if(mRecentRecyclerView == null)
                return;

            mRecentRecyclerView.setVisibility(View.VISIBLE);
            TextViewNoRecords.setVisibility(View.GONE);
        }
    }

    /////////////////////////////////////////////////////////////////////////////////////////
    /////////////////////////////////////////////////////////////////////////////////////////

    public static void LoadMissedCount()
    {
        m_nRecentMissedCount = ReadMissedCount();

        if (m_nRecentMissedCount == 0)
            return;

        MainTabActivity.SetRecentBadgeCount(m_nRecentMissedCount);
    }

    public static void SetMissedCount(int nValue)
    {
        if(nValue == 0)
            m_nRecentMissedCount = 0;
        else
            m_nRecentMissedCount++;

        MainTabActivity.SetRecentBadgeCount(m_nRecentMissedCount);
        StoreMissedCount();
    }

    private static void StoreMissedCount()
    {
        DataPreference.WritePreferenceValue(RECENT_MISSED_COUNT_KEY, m_nRecentMissedCount);
    }

    private static int ReadMissedCount()
    {
        return DataPreference.ReadPreferenceValue(RECENT_MISSED_COUNT_KEY, 0);
    }

    //////////////////////////////////////////////////////////////////////////////
    //////////////////////////////////////////////////////////////////////////////

    public static void PostOpenDialPad(String sCallerNum)
    {
        if (mRecentFragment != null)
            mRecentFragment.OpenDialPad(sCallerNum);
    }

    private void OpenDialPad(String sCallerNum)
    {
        Bundle objBundle = new Bundle();
        objBundle.putString("CallerNum", sCallerNum);

        DialpadFragment objDialpadFragment = new DialpadFragment();
        objDialpadFragment.setArguments(objBundle);

        getActivity().getSupportFragmentManager().beginTransaction().replace(android.R.id.content, objDialpadFragment).addToBackStack(null).commit();
    }
}
