package com.vaxsoft.vaxphonesip.VaxUserAgentSIP.VaxAudioDevice;

import android.content.Context;
import android.media.AudioDeviceInfo;
import android.media.AudioManager;
import java.util.Timer;
import java.util.TimerTask;
import java.util.List;

public class VaxAudioDevice
{
    private Timer m_hTimerTick = null;
    int m_nActivatedDeviceType = -1;

    private Context mContext;

    public VaxAudioDevice(Context objContext)
    {
        mContext = objContext;
    }

    ////////////////////////////////////////////////////////////////////
    ////////////////////////////////////////////////////////////////////

    public void StopTimer()
    {
        if(m_hTimerTick == null)
            return;

        m_hTimerTick.cancel();
        m_hTimerTick = null;
    }

    public void StartTimer()
    {
        if(m_hTimerTick != null)
            return;

        m_hTimerTick = new Timer();
        m_hTimerTick.schedule(new TimerTask()
        {
            @Override
            public void run()
            {
                OnTimerTick();
            }

        }, 0, 2000);
    }

    private void OnTimerTick()
    {
        AudioManager objAudioMgr = (AudioManager) mContext.getSystemService(Context.AUDIO_SERVICE);

        AudioDeviceInfo hAudioDevice = objAudioMgr.getCommunicationDevice();

        if(hAudioDevice.getType() == AudioDeviceInfo.TYPE_BUILTIN_SPEAKER)
            return;

        AudioDeviceInfo hBlueToothDevice = null;
        AudioDeviceInfo hEarPieceDevice = null;

        List<AudioDeviceInfo> hAudioDevices = objAudioMgr.getAvailableCommunicationDevices();

        for (AudioDeviceInfo hAudioDeviceFind : hAudioDevices)
        {
            if (hAudioDeviceFind.getType() == AudioDeviceInfo.TYPE_BLUETOOTH_SCO)
            {
                hBlueToothDevice = hAudioDeviceFind;
                break;
            }

            else if (hAudioDeviceFind.getType() == AudioDeviceInfo.TYPE_BUILTIN_EARPIECE)
            {
                hEarPieceDevice = hAudioDeviceFind;
            }
        }

        if (hBlueToothDevice != null)
        {
            if(m_nActivatedDeviceType == AudioDeviceInfo.TYPE_BLUETOOTH_SCO)
                return;

            objAudioMgr.setCommunicationDevice(hBlueToothDevice);
            m_nActivatedDeviceType = hBlueToothDevice.getType();

            return;
        }

        if(hAudioDevice.getType() == m_nActivatedDeviceType)
            return;

        //objAudioMgr.setBluetoothScoOn(true);
        //objAudioMgr.startBluetoothSco();

        objAudioMgr.setCommunicationDevice(hEarPieceDevice);
        m_nActivatedDeviceType = hEarPieceDevice.getType();
    }

    ////////////////////////////////////////////////////////////////////
    ////////////////////////////////////////////////////////////////////

    public boolean SpeakerPhone(boolean bEnable)
    {
        AudioManager objAudioMgr = (AudioManager) mContext.getSystemService(Context.AUDIO_SERVICE);

        AudioDeviceInfo hBuiltInSpkDevice = null;
        //AudioDeviceInfo hEarPieceDevice = null;

        List<AudioDeviceInfo> hAudioDevices = objAudioMgr.getAvailableCommunicationDevices();

        for (AudioDeviceInfo hAudioDeviceFind : hAudioDevices)
        {
            if (hAudioDeviceFind.getType() == AudioDeviceInfo.TYPE_BUILTIN_SPEAKER)
            {
                hBuiltInSpkDevice = hAudioDeviceFind;
                break;
            }
        }

        if(bEnable)
        {
            //objAudioMgr.setSpeakerphoneOn(true);
            //objAudioMgr.setMode(AudioManager.MODE_IN_COMMUNICATION);

            if(hBuiltInSpkDevice == null)
                return false;

            objAudioMgr.setCommunicationDevice(hBuiltInSpkDevice);
        }
        else
        {
            //objAudioMgr.setSpeakerphoneOn(false);
            //objAudioMgr.setMode(AudioManager.MODE_IN_COMMUNICATION);

            objAudioMgr.clearCommunicationDevice();
        }

        return true;
    }

    public boolean IsSpeakerPhone()
    {
        AudioManager objAudioMgr = (AudioManager) mContext.getSystemService(Context.AUDIO_SERVICE);
        //return objAudioMgr.isSpeakerphoneOn();

        AudioDeviceInfo hAudioDevice = objAudioMgr.getCommunicationDevice();
        boolean bEnabled =  (hAudioDevice.getType() == AudioDeviceInfo.TYPE_BUILTIN_SPEAKER);

        return bEnabled;
    }
}

